==> toycars-0.3.2.tar.gz: toycars-0.3.2/src/Startline.cpp <==
/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 19/04/05.
 */

#include "Startline.h"

#include <algorithm>

#define min4(a,b,c,d)	std::min(std::min((a), (b)), std::min((c), (d)))
#define max4(a,b,c,d)	std::max(std::max((a), (b)), std::max((c), (d)))

Startline::Startline(Tuple inP1, Tuple inP2, double width, bool inDrawArrow)
	: ScPanel(0,0), p1(inP1), p2(inP2), drawArrow(inDrawArrow)
{
	// setup the start line bounds
	setLine(p1, p2, width);
	
	// upload a texture for the startline
	int i, j;
	const int texwidth = 16;
	const int texheight = 16;
	unsigned char pixels[texwidth*texheight*3];
	unsigned char * curpix = pixels;
	for (i = 0; i < texheight; i++)
	{
		for (j = 0; j < texwidth; j++)
		{
			curpix[0] = curpix[1] = curpix[2] = ((i<(texheight/2)) == (j<(texwidth/2))) ? 0 : 255;
			curpix += 3;
		}
	}
	glEnable(GL_TEXTURE_2D);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, texwidth);
	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, texwidth, texheight, 0, GL_RGB, GL_UNSIGNED_BYTE, pixels);
}

Startline::~Startline()
{
	glDeleteTextures(1, &texID);
}

void Startline::setLine(Tuple inP1, Tuple inP2, double width)
{
	Tuple N;
	
	p1 = inP1;
	p2 = inP2;
	N = (0.5*width)*(p2 - p1).perp().Direction();
	pA = p1 + N;
	pB = p2 + N;
	pC = p2 - N;
	pD = p1 - N;
	
	bounds.left = min4(pA.x, pB.x, pC.x, pD.x);
	bounds.bottom = min4(pA.y, pB.y, pC.y, pD.y);
	bounds.right = max4(pA.x, pB.x, pC.x, pD.x);
	bounds.top = max4(pA.y, pB.y, pC.y, pD.y);
}

void Startline::doDraw()
{
	// temp hack
	static double tx2 = (pB - pC).Length() / 16.0;
	static double ty2 = (pA - pB).Length() / 16.0;
	
	glEnable(GL_TEXTURE_2D);
	glColor4ub(255,255,255,255);
	glDisable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, texID);
	glBegin(GL_QUADS);
		glTexCoord2d(0.0, 0.0);	glVertex2d(pA.x, pA.y);
		glTexCoord2d(0.0, ty2);	glVertex2d(pB.x, pB.y);
		glTexCoord2d(tx2, ty2);	glVertex2d(pC.x, pC.y);
		glTexCoord2d(tx2, 0.0);	glVertex2d(pD.x, pD.y);
	glEnd();
	
	if (drawArrow)
	{
		Tuple N = (p2-p1).perp().Direction() * 50.0;
		Tuple M = (p1+p2)*0.5 - N*0.5;
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glColor4f(1.0,1.0,1.0,0.6);
		glBegin(GL_TRIANGLES);
			glVertex2d(M.x, M.y);
			glVertex2d(M.x - N.x - N.y*0.2, M.y - N.y + N.x*0.2);
			glVertex2d(M.x - N.x + N.y*0.2, M.y - N.y - N.x*0.2);
		glEnd();
		glBegin(GL_QUADS);
			glVertex2d(M.x - N.x - N.y*0.05, M.y - N.y + N.x*0.05);
			glVertex2d(M.x - N.x + N.y*0.05, M.y - N.y - N.x*0.05);
			glVertex2d(M.x - 2*N.x + N.y*0.05, M.y - 2*N.y - N.x*0.05);
			glVertex2d(M.x - 2*N.x - N.y*0.05, M.y - 2*N.y + N.x*0.05);
		glEnd();
	}
}

==> toycars-0.3.3.tar.gz: toycars-0.3.3/src/Startline.cpp <==
/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Henner Zilibowitz on 19/04/05.
 */

#include "Startline.h"

#include <algorithm>

#define min4(a,b,c,d)	std::min(std::min((a), (b)), std::min((c), (d)))
#define max4(a,b,c,d)	std::max(std::max((a), (b)), std::max((c), (d)))

Startline::Startline(Tuple inP1, Tuple inP2, double width, bool inDrawArrow)
	: ScPanel(0,0), p1(inP1), p2(inP2), drawArrow(inDrawArrow)
{
	// setup the start line bounds
	setLine(p1, p2, width);
	
	// upload a texture for the startline
	int i, j;
	const int texwidth = 16;
	const int texheight = 16;
	unsigned char pixels[texwidth*texheight*3];
	unsigned char * curpix = pixels;
	for (i = 0; i < texheight; i++)
	{
		for (j = 0; j < texwidth; j++)
		{
			curpix[0] = curpix[1] = curpix[2] = ((i<(texheight/2)) == (j<(texwidth/2))) ? 0 : 255;
			curpix += 3;
		}
	}
	glEnable(GL_TEXTURE_2D);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, texwidth);
	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, texwidth, texheight, 0, GL_RGB, GL_UNSIGNED_BYTE, pixels);
}

Startline::~Startline()
{
	glDeleteTextures(1, &texID);
}

void Startline::setLine(Tuple inP1, Tuple inP2, double width)
{
	Tuple N;
	
	p1 = inP1;
	p2 = inP2;
	N = (0.5*width)*(p2 - p1).perp().Direction();
	pA = p1 + N;
	pB = p2 + N;
	pC = p2 - N;
	pD = p1 - N;
	
	bounds.left = min4(pA.x, pB.x, pC.x, pD.x);
	bounds.bottom = min4(pA.y, pB.y, pC.y, pD.y);
	bounds.right = max4(pA.x, pB.x, pC.x, pD.x);
	bounds.top = max4(pA.y, pB.y, pC.y, pD.y);
}

void Startline::doDraw()
{
	// temp hack
	static double tx2 = (pB - pC).Length() / 16.0;
	static double ty2 = (pA - pB).Length() / 16.0;
	
	glEnable(GL_TEXTURE_2D);
	glColor4ub(255,255,255,255);
	glDisable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, texID);
	glBegin(GL_QUADS);
		glTexCoord2d(0.0, 0.0);	glVertex2d(pA.x, pA.y);
		glTexCoord2d(0.0, ty2);	glVertex2d(pB.x, pB.y);
		glTexCoord2d(tx2, ty2);	glVertex2d(pC.x, pC.y);
		glTexCoord2d(tx2, 0.0);	glVertex2d(pD.x, pD.y);
	glEnd();
	
	if (drawArrow)
	{
		Tuple N = (p2-p1).perp().Direction() * 50.0;
		Tuple M = (p1+p2)*0.5 - N*0.5;
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glColor4f(1.0,1.0,1.0,0.6);
		glBegin(GL_TRIANGLES);
			glVertex2d(M.x, M.y);
			glVertex2d(M.x - N.x - N.y*0.2, M.y - N.y + N.x*0.2);
			glVertex2d(M.x - N.x + N.y*0.2, M.y - N.y - N.x*0.2);
		glEnd();
		glBegin(GL_QUADS);
			glVertex2d(M.x - N.x - N.y*0.05, M.y - N.y + N.x*0.05);
			glVertex2d(M.x - N.x + N.y*0.05, M.y - N.y - N.x*0.05);
			glVertex2d(M.x - 2*N.x + N.y*0.05, M.y - 2*N.y - N.x*0.05);
			glVertex2d(M.x - 2*N.x - N.y*0.05, M.y - 2*N.y + N.x*0.05);
		glEnd();
	}
}
